Ext.namespace('Zarafa.plugins.sin_jitsi');

Zarafa.plugins.sin_jitsi.Plugin = Ext.extend(Zarafa.core.Plugin, {
  
  plugin: undefined,
  
  initPlugin: function(){
    Zarafa.core.data.SharedComponentType.addProperty('plugins.sin_jitsi.panel');
    Zarafa.core.data.SharedComponentType.addProperty('plugins.sin_jitsi.addmeetingdialog');
    this.registerInsertionPoint('context.settings.categories', this.createSettingsCategory, this);
    if(!container.getSettingsModel().get('zarafa/v1/plugins/sin_jitsi/hidetabbarbutton')){
      this.registerInsertionPoint('main.toolbar.actions.last', this.createToolbarButton, this);
    }
    this.registerInsertionPoint('context.calendar.appointmentcontentpanel.toolbar.actions', this.createAddMeetingButton, this);
    this.registerInsertionPoint('context.calendar.appointmentcontentpanel.toolbar.actions', this.createJoinMeetingButton, this);
    this.registerInsertionPoint('context.mail.mailcreatecontentpanel.toolbar.actions', this.createAddMeetingButton, this);
    if(!container.getSettingsModel().get('zarafa/v1/plugins/sin_jitsi/nolocationfix')){
      //Hook into location setting for ressources and prevent removing of the meeting url:
      var Zarafa_calendar_dialogs_AppointmentTab_doSetLocation_or = Zarafa.calendar.dialogs.AppointmentTab.prototype.doSetLocation;
      Zarafa.calendar.dialogs.AppointmentTab.prototype.doSetLocation = function(){
        //Add our jitsi url back if the location was changed automatically
        if('jitsiCurrentUrl' in this.record && arguments[0].indexOf(this.record.jitsiCurrentUrl) < 0){
          if(!container.getSettingsModel().get('zarafa/v1/plugins/sin_jitsi/locationoverride')){
            arguments[0] += (arguments[0] ? ' / ' : '') + this.record.jitsiCurrentUrl;
          }else{
            arguments[0] = this.record.jitsiCurrentUrl;
          }
        }
        Zarafa_calendar_dialogs_AppointmentTab_doSetLocation_or.apply(this, arguments);
      };
      var Zarafa_calendar_dialogs_AppointmentTab_onFieldChange_or = Zarafa.calendar.dialogs.AppointmentTab.prototype.onFieldChange;
      Zarafa.calendar.dialogs.AppointmentTab.prototype.onFieldChange = function(){
        if(arguments[0].getName() == 'location' && 'jitsiCurrentUrl' in this.record && arguments[1].indexOf(this.record.jitsiCurrentUrl) < 0){
          //If the user manually removes the url from the location, prevent adding it again if the location gets set
          delete this.record.jitsiCurrentUrl;
        }
        Zarafa_calendar_dialogs_AppointmentTab_onFieldChange_or.apply(this, arguments);
      };
    }
  }, 
  
  createSettingsCategory: function(){
    return {
      xtype: 'siedl.jitsi.settingscategory',
      plugin: this
    };
  },
  
  createToolbarButton: function(){
		return [{
			newMenuIndex: 10,
			xtype: 'button',
			scale: 'large',
			tooltip: 'Jitsi Webmeetings',
			iconCls: 'icon_sin_jitsi_32',
			handler: this.onToolbarButtonClick,
			scope: this,
			}];
	},
  
  createAddMeetingButton: function(){
    return {
      xtype: 'button',
      text: _('Add webmeeting', 'plugin_sin_jitsi'),
      iconCls: 'icon_sin_jitsi',
      handler: this.onAddMeetingButtonClick,
      scope: this,
      plugins: ['zarafa.recordcomponentupdaterplugin'],
      tooltip: _('Shift-click for advanced settings', 'plugin_sin_jitsi'),
      update: function(record, contentReset){
        this.record = record;
        if ((new RegExp(container.getSettingsModel().get('zarafa/v1/plugins/sin_jitsi/server').replace(/[.*+?^${}()|[\]\\]/g, '\\$&') + '[-a-zA-Z0-9()@:%_\+.~#?&\/=]+')).test(record.get('location'))) {
          this.setDisabled(true);
        } else {
          this.setDisabled(false);
        }
      }
    };
  },
  
  createJoinMeetingButton: function(){
    return {
      xtype: 'button',
      text: _('Join webmeeting', 'plugin_sin_jitsi'),
      iconCls: 'icon_sin_jitsi',
      handler: this.onJoinMeetingButtonClick,
      scope: this,
      plugins: ['zarafa.recordcomponentupdaterplugin'],
      update: function(record, contentReset){
        this.record = record;
        if (/https?:\/\/[a-zA-Z0-9-\.]+(?:\/[-a-zA-Z0-9()@:%_\+.~#?&\/=]*)?/.test(record.get('location'))) {
          this.setDisabled(false);
        } else {
          this.setDisabled(true);
        }
      }
    };
  },
  
  onAddMeetingButtonClick: function(button,  evt){
    if(evt.shiftKey){
      Zarafa.core.data.UIFactory.openLayerComponent(Zarafa.core.data.SharedComponentType['plugins.sin_jitsi.addmeetingdialog'], null, {modal : true, source: button});
      return
    }
    //generate room url
    var hash = 0, i, chr;
    var agent = container.getUser().getUserName() + navigator.userAgent + (new Date()).valueOf();
    for (i = 0; i < agent.length; i++) {
      chr = agent.charCodeAt(i);
      hash = ((hash << 5) - hash) + chr;
      hash |= 0;
    }
    if(hash < 0) hash += 2147483647;
    var mname = '';
    if(!container.getSettingsModel().get('zarafa/v1/plugins/sin_jitsi/mname_nosubject')){
      mname += (button.record.get('subject') || 'jitsi').replace(/[^a-zA-Z0-9\-]/g, '_').replace(/_{2,}/g,  '_').replace(/^_+|_+$/g, '') + '-';
    }
    if(!container.getSettingsModel().get('zarafa/v1/plugins/sin_jitsi/mname_noorganizer')){
      mname += container.getUser().getUserName() + '-';
    }
    button.record.jitsiCurrentUrl = container.getSettingsModel().get('zarafa/v1/plugins/sin_jitsi/server') + mname + hash.toString(16);
    //modify record depending on type
    var edf = null;
    switch(button.record.get('message_class')){
      case 'IPM.Note': //Mail
        edf = button.ownerCt.ownerCt.editorField;
        break;
      case 'IPM.Appointment':
        var oloc = button.record.get('location');
        if(oloc && !container.getSettingsModel().get('zarafa/v1/plugins/sin_jitsi/locationoverride')){
          button.record.set('location', oloc + ' / ' + button.record.jitsiCurrentUrl);
        }else{
          button.record.set('location', button.record.jitsiCurrentUrl);
        }
        if(container.getSettingsModel().get('zarafa/v1/plugins/sin_jitsi/noinvitation')){
          return;
        }
        edf = button.ownerCt.ownerCt.appointmentTab.editorField;
        break
    }
    //Add invitation to item body
    if(edf){
      var iurl = button.record.jitsiCurrentUrl;
      var itpl = container.getSettingsModel().get('zarafa/v1/plugins/sin_jitsi/invitationmessage') || '\n%url%\n';
      if(edf.isHtmlEditor()){
        iurl = '<a href=' + iurl + '>' + iurl + '</a>';
        itpl = itpl.replace(/(\r\n|\n|\r)/gm, '<br/>');
      }
      edf.insertAtCursor(itpl.replace(/%url%/g, iurl));
    }
  },
  
  onJoinMeetingButtonClick: function(button){
    var murl = /https?:\/\/[a-zA-Z0-9-\.]+(?:\/[-a-zA-Z0-9()@:%_\+.~#?&\/=]*)?/.exec(button.record.get('location'))
    if(murl){
      this.openMeeting(murl[0], button.record.get('subject'));
    }
  },
  
  onToolbarButtonClick: function(){
    this.openMeeting(container.getSettingsModel().get('zarafa/v1/plugins/sin_jitsi/server'), 'Jitsi');
  }, 
  
  openMeeting: function(meetingurl, meetingname){
    switch(container.getSettingsModel().get('zarafa/v1/plugins/sin_jitsi/openin')){
      case 'browser':
        window.open(meetingurl);
        break;
      case 'popup':
        if(window.deskappOpenWindow){
          window.deskappOpenWindow(meetingurl, function(windowInstance){}, {height:768,width:1024});
        }else{
          window.open(meetingurl, '_blank', 'location=no,height=768,width=1024,scrollbars=yes,status=yes');
        }
        break;
      default:
        Zarafa.core.data.UIFactory.openLayerComponent(
          Zarafa.core.data.SharedComponentType['plugins.sin_jitsi.panel'],
          null,
          {
            url: meetingurl,
            title: 'Meeting: ' + meetingname,
            iconCls : 'icon_sin_jitsi',
            tabOrder: (new Date()).valueOf() //We dont need a reference to the tab anyways... TODO properly if i finally write a intranet-plugin with extended plugin capabilities...
          }
        );
        break;
    }
  }, 
  
  //Stuff for iframe tabs
  bidSharedComponent: function(type, record){
      var bid = -1;
      switch(type){
        case Zarafa.core.data.SharedComponentType['plugins.sin_jitsi.panel']:
          bid = 1;
          break;
        case Zarafa.core.data.SharedComponentType['plugins.sin_jitsi.addmeetingdialog']:
          bid = 1;
          break;
      }
      return bid;
  },
  
  getSharedComponent: function(type, record){
    var component;
    switch(type){
      case Zarafa.core.data.SharedComponentType['plugins.sin_jitsi.panel']:
        return Zarafa.plugins.sin_jitsi.ui.ContentPanel;
      case Zarafa.core.data.SharedComponentType['plugins.sin_jitsi.addmeetingdialog']:
        return Zarafa.plugins.sin_jitsi.ui.AddMeetingDialog;
    }
  },
  
});

Zarafa.onReady(function(){
  container.registerPlugin(new Zarafa.core.PluginMetaData({
    name: 'sin_jitsi',
    displayName: 'Jitsi Webmeetings',
    //about: Zarafa.plugins.sin_jitsi.ABOUT,
    pluginConstructor: Zarafa.plugins.sin_jitsi.Plugin
  }));
});