Ext.namespace('Zarafa.plugins.sin_jitsi.ui');

//Originally from the kopano intranet plugin, with minor modifications:
/**
 * @class Zarafa.plugins.sin_jitsi.ui.ContentPanel
 * @extends Zarafa.core.ui.ContentPanel
 */
Zarafa.plugins.sin_jitsi.ui.ContentPanel = Ext.extend(Zarafa.core.ui.ContentPanel, {
  /**
   * @constructor
   * @param config Configuration structure
   */
  constructor : function(config)
  {
    config = config || {};

    Ext.applyIf(config, {
      // Overridden from Ext.Component
      xtype: 'sin_jitsi_contentpanel',
      layout : 'fit',
      iconCls: config.iconCls,
      border: false,
      items : [{
        xtype: 'sin_jitsi_panel',
        url: config.url,
        tabOrder: config.tabOrder
      }]
    });

    Zarafa.plugins.sin_jitsi.ui.ContentPanel.superclass.constructor.call(this, config);
  }
});
Ext.reg('sin_jitsi_contentpanel', Zarafa.plugins.sin_jitsi.ui.ContentPanel);

/**
 * @class Zarafa.plugins.sin_jitsi.ui.Panel
 * @extends Ext.Panel
 */
Zarafa.plugins.sin_jitsi.ui.Panel = Ext.extend(Ext.Panel, {
  /**
   * The id of the iframe element inside this panel
   */
  iframeId : undefined,

  /**
   * @cfg {Boolean} isLoadMaskShown true if load mask should be shown else false.
   */
  isLoadMaskShown : false,

  /**
   * The LoadMask object which will be shown when the {@link #record} is being opened, and
   * the dialog is waiting for the server to respond with the desired data. This will only
   * be set if {@link #showLoadMask} is true.
   * @property
   * @type Zarafa.common.ui.LoadMask
   */
  loadMask : undefined,

  /**
   * @constructor
   * @param {Object} config Configuration structure
   */
  constructor : function(config)
  {
    config = config || {};
    this.iframeId = 'intranet-iframe-'+config.tabOrder;
    if (Ext.isFunction(Zarafa.isDeskApp) ? Zarafa.isDeskApp() : Ext.isDefined(window.nw)) {
      this.tag = 'webview';
    } else {
      this.tag = 'iframe';
    }

    Ext.applyIf(config, {
      // Overridden from Ext.Component
      xtype: 'sin_jitsi_panel',
      layout : 'fit',
      header: false,
      html : {
        tag: this.tag,
        id: this.iframeId,
        cls: 'sin_jitsi-iframe',
        src: config.url,
        style: 'display:block',
        //Geri allow mic & cam if iframe!
        allow: 'microphone *; camera *; display-capture *;'
      },
      listeners: {
        afterrender: this.onAfterRender,
        scope: this
      }
    });

    Zarafa.plugins.sin_jitsi.ui.Panel.superclass.constructor.call(this, config);
  },

  /**
   * Handler for the afterrender event of this panel. Will set a load mask when opening
   * a url. When using WEBVIEW (i.e. in DeskApp) it will add some listeners to handle
   * dialogs in the webview.
   */
  onAfterRender: function()
  {
    this.showLoadMask();

    var iframe = document.getElementById(this.iframeId);
    var event = (this.tag === 'webview') ? 'contentload' : 'load';

/*
    if(this.tag === 'webview' && typeof navigator.mediaDevices.getDisplayMedia === 'undefined'){ //DeskApp
      // Inject custom getDisplayMedia function to support screensharing
      iframe.addEventListener('loadcommit',  function(){
        var inj = `
          navigator.mediaDevices.getDisplayMedia = function(){
            console.log(arguments);
          }
          function sin_setOrigin(event){
            if(event.data !== 'sin_setOrigin') return;
            window.sin_origin = event.source;
            event.source.postMessage('receivedOrigin',  '*');
            window.removeEventListener('message', sin_setOrigin, false);
          }
          window.addEventListener('message', sin_setOrigin, false);
        `
        iframe.addEventListener('message', console.log);
        iframe.executeScript({code: inj, mainWorld: true});
        // iframe.contentWindow.postMessage('sin_setOrigin', '*');
      })
    }
*/

    iframe.addEventListener(event, function(){
      this.hideLoadMask();
      // For DeskApp: copy cookies to the main context so files can be downloaded
      if (this.tag === 'webview') {
        var intraUrl = this.url;
        chrome.cookies.getAll(
          {
            url: intraUrl,
            storeId: iframe.getCookieStoreId()
          },
          function(cookies) {
            for (var cookie of cookies) {
              chrome.cookies.set({
                url: intraUrl,
                domain: cookie.domain,
                expirationDate: cookie.expirationDate,
                httpOnly: cookie.httpOnly,
                name: cookie.name,
                path: cookie.path,
                sameSite: cookie.sameSite,
                secure: cookie.secure,
                value: cookie.value,
              });
            }
          }
        );
      }

    }.createDelegate(this));

    // Add some event listeners when we are using webviews (i.e. in DeskApp)
    if ( this.tag === 'webview' ){
      iframe.addEventListener('permissionrequest', this.handlePermissionRequests);
      iframe.addEventListener('dialog', this.handleDialogRequests, this);
      iframe.addEventListener('newwindow', this.handleNewWindowRequests, this);

    }
  },

  /**
   * Handler for the dialog event of WEBVIEW elements. Will handle alert, prompt,
   * and confirm dialogs
   * @param  {Event} e The dialog event
   */
  handleDialogRequests : function(e)
  {
    // Handle alerts
    if ( e.messageType === 'alert' ) {
      window.alert(e.messageText);
    }

    // Handle confirm dialogs
    else if ( e.messageType === 'confirm' ) {
      var confirmation =  window.confirm(e.messageText);

      if ( confirmation ) {
        e.dialog.ok();
      } else {
        e.dialog.cancel();
      }
    }

    // Handle prompts
    else if ( e.messageType === 'prompt' ){
      var wprompt = window.prompt( e.messageText);

      if ( wprompt === null ){
        e.dialog.cancel();
      } else {
        e.dialog.ok(wprompt);
      }
    }

  },

  /**
   * Handler for the permissionrequest event of WEBVIEW elements. Will handle the request
   * by its type.
   * Possible types are media, geolocation, pointerLock, download, loadplugin and fullscreen.
   * For now we deny geolocation, fullscreen and pointerLock requests.
   * @param {Event} e The permissionrequest event
   */
  handlePermissionRequests : function(e)
  {
    e.preventDefault();
    switch (e.permission) {
      // Allow
      case 'download':
      case 'media':
      case 'loadplugin':
        e.request.allow();
      break;
      // Deny
      case 'pointerLock':
      case 'fullscreen':
      case 'geolocation':
        e.request.deny();
      break;
      // also deny all other, not yet known, requests
      default:
        e.request.deny();
      break;

    }
  },

  /**
   * Handler for the newwindow event of WEBVIEW elements. Will handle new windows, by
   * opening them externally in the browser.
   * @param  {Event} e The newwindow event
   */
  handleNewWindowRequests : function(e)
  {
    e.window.discard();
    nw.Shell.openExternal(e.targetUrl);
  },

  /**
   * If {@link #showLoadMask} is enabled, this function will display
   * the {@link #loadMask}.
   * @param {Boolean} errorMask True to show an error mask instead of the loading mask.
   * @protected
   */
  showLoadMask : function(errorMask)
  {
    if (this.isLoadMaskShown === true) {
      return;
    }
    if (!this.loadMask) {
      this.loadMask = new Zarafa.common.ui.LoadMask(this.ownerCt.el);
    }

    if (errorMask) {
      this.loadMask.showError();
    } else {
      this.loadMask.show();
      this.isLoadMaskShown = true;
    }
  },

  /**
   * If {@link #showLoadMask} is enabled, and {@link #showLoadMask} has been
   * called to display the {@link #loadMask} this function will disable the
   * loadMask.
   * @protected
   */
  hideLoadMask : function()
  {
    if (this.isLoadMaskShown === false) {
      return;
    }

    if (this.loadMask) {
      this.loadMask.hide();
      this.isLoadMaskShown = false;
    }
  }

});
Ext.reg('sin_jitsi_panel', Zarafa.plugins.sin_jitsi.ui.Panel);
